{% extends "admin/change_form.html" %}

{% load i18n %}

{% block submit_buttons_top %}
    {% include "simple_history/submit_line.html" %}
{% endblock %}

{% block submit_buttons_bottom %}
    {% include "simple_history/submit_line.html" %}
{% endblock %}

{% block form_top %}
    <p class="px-3 py-3 rounded-default text-sm last:mb-8 bg-blue-50 mb-8 text-blue-500 dark:bg-blue-500/20">
        {% if not revert_disabled %}
            {% blocktrans %}Press the 'Revert' button below to revert to this version of the object.{% endblocktrans %}
        {% endif %}

        {% if change_history %}
            {% blocktrans %}Press the 'Change History' button below to edit the history.{% endblocktrans %}
        {% endif %}
    </p>
{% endblock %}
