{% if actions_list or actions_detail or actions_items or nav_global %}
    <ul class="flex flex-col font-medium mb-4 ml-auto mt-2 md:flex-row md:mb-0 md:mt-0 max-md:w-full">
        {% if actions_items %}
            {{ actions_items }}
        {% endif %}

        {% if nav_global %}
            {{ nav_global }}
        {% endif %}

        {% for action in actions_list %}
            {% include "unfold/helpers/tab_action.html" with title=action.title link=action.path %}
        {% endfor %}

        {% for action in actions_detail %}
            {% include "unfold/helpers/tab_action.html" with title=action.title link=action.path %}
        {% endfor %}
    </ul>
{% endif %}
