{% load i18n %}

{% if not is_popup %}
    {% if tabs_list or inlines_list or actions_list or actions_detail or actions_items or nav_global %}
        <div class="flex items-start flex-col mb-4 md:flex-row md:items-center">
            {% include "unfold/helpers/tab_items.html" %}

            {% include "unfold/helpers/tab_actions.html" %}
        </div>
    {% endif %}
{% endif %}
