<button type="submit" {% if name %}name="{{ name}}"{% endif %} class="bg-primary-600 block border border-transparent cursor-pointer font-medium px-3 py-2 rounded-default self-end text-sm text-white">
    {{ title }}
</button>
