{% extends "admin/base_site.html" %}
{% load admin_urls i18n static %}

{% block extrastyle %}{{ block.super }}">{% endblock %}

{% block content %}
    <div id="content-main">
        <p class="bg-primary-100 mb-8 text-primary-600 px-3 py-3 rounded-default text-sm">
            {% translate 'Your password was changed.' %}
        </p>
    </div>
{% endblock %}
